//
//  CB_Filter_BVM.java
//  CBSim
//
//  Created by Loren Petrich on 8/7/08.
//  Copyright 2008 __MyCompanyName__. All rights reserved.
//

public class CB_Filter_BVM extends CB_Filter
{
	// Transformation matrices:
	// RGB <-> LMS
	public static final double[][] RGBtoLMS =
	{
		{0.05059983, 0.08585369, 0.00952420},
		{0.01893033, 0.08925308, 0.01370054},
		{0.00292202, 0.00975732, 0.07145979}
	};
	public static final double[][] LMStoRGB =
	{
		{30.830854, -29.832659, 1.610474},
		{-6.481468, 17.715578, -2.532642},
		{-0.375690, -1.199062, 14.273846}
	};
	public static final double[][] AnchorPoints =
	{
		{0.08008, 0.1579, 0.5897},
		{0.1284, 0.2237, 0.3636},
		{0.9856, 0.7325, 0.001079},
		{0.0914, 0.007009, 0.0}
	};
	
	private static double[] White = new double[3];
	private static double[][][] Params = new double[3][2][3];
	
	public static void CrossProduct(double[] x, double[] y, double[] z)
	{
		z[0] = x[1]*y[2] - x[2]*y[1];
		z[1] = x[2]*y[0] - x[0]*y[2];
		z[2] = x[0]*y[1] - x[1]*y[0];
	}
	
	static
	{
		for (int i=0; i<3; i++)
		{
			double[] Row = RGBtoLMS[i];
			double sum = 0;
			for (int j=0; j<3; j++)
				sum += Row[j];
			White[i] = sum;
		}
		
		CrossProduct(White, AnchorPoints[2], Params[0][0]);
		CrossProduct(White, AnchorPoints[0], Params[0][1]);
		CrossProduct(White, AnchorPoints[2], Params[1][0]);
		CrossProduct(White, AnchorPoints[0], Params[1][1]);
		CrossProduct(White, AnchorPoints[3], Params[2][0]);
		CrossProduct(White, AnchorPoints[1], Params[2][1]);
		
		// Pre-divide:
		for (int i=0; i<3; i++)
		{
			double[][] Params1 = Params[i];
			for (int j=0; j<2; j++)
			{
				double[] Params2 = Params1[j];
				double Params3 = Params2[i];
				for (int k=0; k<3; k++)
					Params2[k] /= Params3;
			}
		}
	}
	
	private double[] ClrTfm = new double[3];
	private double[][] ClrDog = new double[2][3];
	
	public void SetTransformType(int cbtype_)
	{
		// No setup needed
	}
	
	// Uses previously-set type
	public void DoTransform(double[] ClrIn, double[] ClrOut)
	{
		// The RGB <-> LMS transform is out here, so it only has to be done once for dog vision
		for (int i=0; i<3; i++)
		{
			double[] Row = RGBtoLMS[i];
			double sum = 0;
			for (int j=0; j<3; j++)
				sum += Row[j]*ClrIn[j];
			ClrTfm[i] = sum;
		}
		
		int cbtype = getType();
		if (cbtype < 3)
			DoTransform(cbtype, ClrTfm);
		else
		{
			// Do dog vision by averaging
			for (int i=0; i<3; i++)
				ClrDog[0][i] = ClrDog[1][i] = ClrTfm[i];
			for (int k=0; k<2; k++)
				DoTransform(k, ClrDog[k]);
			for (int i=0; i<3; i++)
				ClrTfm[i] = (ClrDog[0][i] + ClrDog[1][i])/2;
		}
		
		for (int i=0; i<3; i++)
		{
			double[] Row = LMStoRGB[i];
			double sum = 0;
			for (int j=0; j<3; j++)
				sum += Row[j]*ClrTfm[j];
			ClrOut[i] = sum;
		}
	}
	
	// Valid only for 0, 1, or 2 (absent channels: no-red, no-green, or no-blue)
	private void DoTransform(int cbtype, double[] ClrTfm)
	{
		
		// The two channels other than the absent one
		int ilo = 0;
		if (ilo == cbtype) ilo++;
		int ihi = 2;
		if (ihi == cbtype) ihi--;
		
		// Which parameters?
		double[][] Params1 = Params[cbtype];
		// Which side of the white point?
		double[] Params2 =
			(ClrTfm[ihi]*White[ilo] - ClrTfm[ilo]*White[ihi] > 0) ?
				Params1[0] : Params1[1];
		// Division by Params2[cbtype] moved to static block for speed
		ClrTfm[cbtype] = - (ClrTfm[ilo]*Params2[ilo]+ClrTfm[ihi]*Params2[ihi]);		
	}
}
