//
//  CB_Filter_MGWW.java
//  CBSim
//
//  Created by Loren Petrich on 8/7/08.
//  Copyright 2008 __MyCompanyName__. All rights reserved.
//

public class CB_Filter_MGWW extends CB_Filter

{
	// Transformation matrices:
	// RGB <-> XYZ
	public static final double[][] RGBtoXYZ =
	{
		{0.430574, 0.341550, 0.178325},
		{0.222015, 0.706655, 0.071330},
		{0.020183, 0.129553, 0.939180}
	};
	public static final double[][] XYZtoRGB =
	{
		{3.063218, -1.393325, -0.475802},
		{-0.969243, 1.875966, 0.041555},
		{0.067871, -0.228834, 1.069251}
	};
	
	public static final double[][] ConfusionPoints = 
	{
		{0.735, 0.265},
		{1.14, -0.14},
		{0.171, -0.003}
	};
	
	public static final double[][][] AnchorPoints =
	{
		{{0.115807, 0.073581}, {0.471899, 0.527051}},
		{{0.102776, 0.102864}, {0.505845, 0.493211}},
		{{0.045391, 0.294976}, {0.665764, 0.334011}}
	};
	
	private static double[][] AnchorSlopesIntercepts =
		new double[3][2];
	
	// For multiplying the results to get grayscale correct
	public static final double[][] WhitePointFixers =
	{
		{0.963109, 1.011506, 1.00658},
		{0.823265, 1.073951, 0.986204},
		{0.977392, 1.023442, 0.865868},
		{0.887713, 1.041794, 0.996286}
	};
	
	static
	{
		// Set up the anchor points' slopes and intercepts
		for (int i=0; i<3; i++)
		{
			double[][] PtSet = AnchorPoints[i];
			double[] SlpItc = AnchorSlopesIntercepts[i];
			SlpItc[0] = (PtSet[1][1] - PtSet[0][1])/
				(PtSet[1][0] - PtSet[0][0]);
			SlpItc[1] = PtSet[0][1] - SlpItc[0]*PtSet[0][0];
		}
	}
	
	private double[] ClrTfm = new double[3];
	private double[][] ClrDog = new double[2][3];
	
	public void SetTransformType(int cbtype_)
	{
		// No setup needed
	}
	
	// Uses previously-set type
	public void DoTransform(double[] ClrIn, double[] ClrOut)
	{
		// The RGB <-> LMS transform is out here, so it only has to be done once for dog vision
		for (int i=0; i<3; i++)
		{
			double[] Row = RGBtoXYZ[i];
			double sum = 0;
			for (int j=0; j<3; j++)
				sum += Row[j]*ClrIn[j];
			ClrTfm[i] = sum;
		}
		
		int cbtype = getType();
		if (cbtype < 3)
			DoTransform(cbtype, ClrTfm);
		else
		{
			// Do dog vision by averaging
			for (int i=0; i<3; i++)
				ClrDog[0][i] = ClrDog[1][i] = ClrTfm[i];
			for (int k=0; k<2; k++)
				DoTransform(k, ClrDog[k]);
			for (int i=0; i<3; i++)
				ClrTfm[i] = (ClrDog[0][i] + ClrDog[1][i])/2;
		}
	
		for (int i=0; i<3; i++)
		{
			double[] Row = XYZtoRGB[i];
			double sum = 0;
			for (int j=0; j<3; j++)
				sum += Row[j]*ClrTfm[j];
			ClrOut[i] = sum;
		}
		
		double[] WPF = WhitePointFixers[cbtype];
		for (int i=0; i<3; i++)
			ClrOut[i] *= WPF[i];
	}

	// Valid only for 0, 1, or 2 (absent channels: no-red, no-green, or no-blue)
	private void DoTransform(int cbtype, double[] ClrTfm)
	{
		double xyzsum = ClrTfm[0] + ClrTfm[1] + ClrTfm[2];
		if (xyzsum == 0) return; // Don't do anything for black
		double u = ClrTfm[0]/xyzsum;
		double v = ClrTfm[1]/xyzsum;
		
		double[] ConfPt = ConfusionPoints[cbtype];
		double clslp = (v - ConfPt[1])/(u - ConfPt[0]);
		double clitc = v - clslp*u;
		
		double[] SlpItc = AnchorSlopesIntercepts[cbtype];
		u = - (SlpItc[1] - clitc)/(SlpItc[0] - clslp);
		v = clitc + (clslp*u);
		
		ClrTfm[0] = (u*ClrTfm[1])/v;
		ClrTfm[2] = ((1 - u - v)*ClrTfm[1])/v;
	}
}
