//
//  ImageExtrasDialog.java
//  CBSim
//
//  Created by Loren Petrich on 5/23/08.
//  Copyright 2008 __MyCompanyName__. All rights reserved.
//

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

class ImageExtrasDialog
{
	// Can get and set without anything special;
	// the dialog-box widgets only get manipulated
	// when showing and hiding this dialog box
	double fraction = 1.0;
	double gamma = 1.0;
	int algo = 0;
	int off_edge = 0;
	
	// Persistent
	private JDialog Dlg;
	
	// Used internally
	private static String PartialTextTitle = "Partial Color Blindness";
	private static String[] PartialTextLabels =
		{"None: 0, Complete: 1"};
	private TextFieldPane PartialText =
		new TextFieldPane(PartialTextTitle, PartialTextLabels);
	
	private static String GammaTextTitle = "Gamma Correction";
	private static String[] GammaTextLabels =
		{"(Stored value) = (True value)^(1/gamma)",
		"(True value) = (Stored value)^(gamma)",
		"No correction: gamma = 1"};
	private TextFieldPane GammaText =
		new TextFieldPane(GammaTextTitle, GammaTextLabels);
	
	private static String AlgoPaneTitle = "Simulation Algorithm";
	private static final String[] Algorithms =
		{"Linear", "Brettel-Vienot-Mollon", "Meyer-Greenberg-Wolfmaier-Wickline"};
	private RadioButtonPane AlgoRBs =
		new RadioButtonPane(AlgoPaneTitle, Algorithms, algo);
	
	private static final String OffEdgePaneTitle = "Off-Edge Colors";
	private static final String[] OffEdgeActions =
		{"Clamp Individually", "Push to Gray Value"};
	private RadioButtonPane OffEdgeRBs =
		new RadioButtonPane(OffEdgePaneTitle, OffEdgeActions, off_edge);
	
	private boolean IsOK = false;
	
	private double fraction_ = fraction;
	private double gamma_ = gamma;
	private NumberChecker NumChk = new NumberChecker();
	private RangeChecker RngChk = new RangeChecker();
	private NR_Checker NR_Chk = new NR_Checker();
	
	ImageExtrasDialog()
	{
		Dlg = new JDialog((JFrame)null, "Extras", true);
		Dlg.setResizable(false);
		Container CPn = Dlg.getContentPane();
		JPanel Ctnts = new JPanel();
		CPn.add(Ctnts);
		
		int BSize = 8;
		Ctnts.setLayout(new BoxLayout(Ctnts, BoxLayout.Y_AXIS));
		Ctnts.setBorder(BorderFactory.createEmptyBorder(BSize,BSize,BSize,BSize));
		
		Ctnts.add(PartialText);
		Ctnts.add(GammaText);
		Ctnts.add(AlgoRBs);
		Ctnts.add(OffEdgeRBs);
		
		JPanel ButtonsPane = new JPanel();
		ButtonsPane.setLayout(new BoxLayout(ButtonsPane, BoxLayout.X_AXIS));
	
		ButtonsPane.add(Box.createHorizontalGlue());
		
		JButton CancelButton = new JButton("Cancel");
		CancelButton.addActionListener(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					Dlg.setVisible(false);
					IsOK = false;
				}
			}
		);
		ButtonsPane.add(CancelButton);
		
		JButton OK_Button = new JButton("OK");
		OK_Button.addActionListener(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					if (!GetValues()) return;
					
					fraction = fraction_;
					gamma = gamma_;
					algo = AlgoRBs.getSelected();
					off_edge = OffEdgeRBs.getSelected();
					
					Dlg.setVisible(false);
					IsOK = true;
				}
			}
		);
		ButtonsPane.add(OK_Button);
		
		Ctnts.add(ButtonsPane);
		
		JRootPane RtPn = Dlg.getRootPane();
		RtPn.setDefaultButton(OK_Button);
		KeyStroke EscapeKeystroke = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0, false);
		RtPn.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(EscapeKeystroke, "ESCAPE");
		RtPn.getActionMap().put("ESCAPE",
			new AbstractAction()
			{
				public void actionPerformed(ActionEvent e)
				{
					Dlg.setVisible(false);
				}
			}
		);
		
		Dlg.pack();
	}
	
	// Returns true for OK, false for cancel
	boolean show()
	{
		IsOK = false;
		fraction_ = fraction;
		gamma_ = gamma;
		
		PartialText.setText(String.valueOf(fraction_));
		GammaText.setText(String.valueOf(gamma_));
		AlgoRBs.setSelected(algo);
		OffEdgeRBs.setSelected(off_edge);
		
		Dlg.setVisible(true);
		
		return IsOK;
	}
	
	private boolean GetValues()
	{
		boolean AllOK = true;
		
		if (NR_Chk.Get(PartialText, 0.0, 1.0, "color-blindness fraction"))
			fraction_ = NR_Chk.value;
		else
			AllOK = false;
		
		if (NR_Chk.Get(GammaText, 1.0, 2.5, "gamma"))
			gamma_ = NR_Chk.value;
		else
			AllOK = false;
		
		return AllOK;
	}
	
	static void ShowError(String ErrStr)
	{
		JOptionPane.showMessageDialog(null, ErrStr, "CBSim Message", JOptionPane.ERROR_MESSAGE);
	}
	
	// Made inner classes so as not to clutter the global namespace
	
	private class NumberChecker
	{
		double value = 0;
		
		boolean Get(String Input, String WhatKind)
		{
			try
			{
				value = Double.parseDouble(Input);
			}
			catch (Exception ex)
			{
				ShowError(String.format("Bad numerical value for %s: %s", WhatKind, Input));
				return false;
			}
			return true;
		}
	}
	
	private class RangeChecker
	{
		double value = 0;
		
		boolean Get(double inpval, double minval, double maxval, String WhatKind)
		{
			boolean IsOK = true;
			value = inpval;
			
			if (value < minval)
			{
				value = minval;
				IsOK = false;
			}
			
			if (value > maxval)
			{
				value = maxval;
				IsOK = false;
			}
			
			if (!IsOK)
				ShowError(String.format("Value of %s is out of range (%s,%s): %s",
					WhatKind, String.valueOf(minval), String.valueOf(maxval), String.valueOf(inpval)));
			
			return IsOK;
		}
	}
	
	private class NR_Checker
	{
		double value = 0;
		
		// Jumping ahead to a text field, since that's what we'll be adjusting
		boolean Get(TextFieldPane TxtFid, double minval, double maxval, String WhatKind)
		{
			if (NumChk.Get(TxtFid.getText(), WhatKind))
			{
				value = NumChk.value;
				
				if (RngChk.Get(value, minval, maxval, WhatKind))
					return true;
				else
				{
					value = RngChk.value;
					TxtFid.setText(String.valueOf(value));
					return false;
				}
			}
			else
				return false;
		}
	}
	
	private class TitledPane extends JPanel
	{
		TitledPane(String Title)
		{
			setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
			setBorder(BorderFactory.createTitledBorder(Title));
			setAlignmentX(Component.LEFT_ALIGNMENT);
		}
	}
	
	private class TextFieldPane extends TitledPane
	{
		private JTextField TxtFld;
		
		TextFieldPane(String Title, String[] Labels)
		{
			super(Title);
			
			for (String Lbl: Labels)
				add(new JLabel(Lbl));
			
			TxtFld = new JTextField(12);
			TxtFld.setMaximumSize(TxtFld.getPreferredSize());
			add(TxtFld);
		}
		
		String getText() {return TxtFld.getText();}
		void setText(String Text) {TxtFld.setText(Text);}
	}
	
	private class RadioButtonPane extends TitledPane
	{
		private JRadioButton Buttons[];
		
		RadioButtonPane(String Title, String[] Labels, int Selected)
		{
			super(Title);
			
			ButtonGroup Group = new ButtonGroup();
			Buttons = new JRadioButton[Labels.length];
			for (int k=0; k<Labels.length; k++)
			{
				JRadioButton Btn = new JRadioButton(Labels[k]);
				if (k == Selected) Btn.setSelected(true);
				Buttons[k] = Btn;
				Group.add(Btn);
				add(Btn);
			}
		}
		
		int getSelected()
		{
			for (int k=0; k<Buttons.length; k++)
				if (Buttons[k].isSelected()) return k;
			return -1;
		}
		
		void setSelected(int Selected) {Buttons[Selected].setSelected(true);}
	}
}
