//
//  ImageOpenFileHandler.java
//  CBSim
//
//  Created by Loren Petrich on 5/20/08.
//  Copyright 2008 __MyCompanyName__. All rights reserved.
//

import java.awt.*;
import java.beans.*;
import java.io.*;
import javax.swing.*;
import javax.swing.filechooser.*;

// Handles reading of image files with a file chooser
public class ImageOpenFileHandler
{
	private JFileChooser Chooser;
	
	public ImageOpenFileHandler()
	{
		ImageFileChooserFilter IFCF = new ImageFileChooserFilter();
		
		Chooser = new JFileChooser();
		Chooser.addChoosableFileFilter(IFCF);
		Chooser.setDialogTitle("Open Image File");
		
		ThumbnailPreview Preview = new ThumbnailPreview();
		Chooser.setAccessory(Preview);
		Chooser.addPropertyChangeListener(Preview);
	}
	
	// Returns null if no file was chosen
	public File GetFile()
	{
		int ReturnValue = Chooser.showOpenDialog(null);
		return (ReturnValue == JFileChooser.APPROVE_OPTION) ?
			Chooser.getSelectedFile() :
				null;
	}
	
	class ThumbnailPreview extends JPanel implements PropertyChangeListener
	{
		private JLabel Thumbnail;
		
		ThumbnailPreview()
		{
			int OverallSize = 72;
			Dimension OverallDim = new Dimension(OverallSize,OverallSize);
			setPreferredSize(OverallDim);
			Thumbnail = new JLabel();
			add(Thumbnail);
		}
		
		public void propertyChange(PropertyChangeEvent e)
		{
			File file = null;
			boolean DoUpdate = false;
			String Property = e.getPropertyName();
			
			if (JFileChooser.DIRECTORY_CHANGED_PROPERTY.equals(Property))
			{
				DoUpdate = true;
			}
			else if (JFileChooser.SELECTED_FILE_CHANGED_PROPERTY.equals(Property))
			{
				file = (File)e.getNewValue();
				DoUpdate = true;
			}
			
			if (!DoUpdate) return;
			if (file == null)
			{
				Thumbnail.setIcon(null);
				return;
			}
			
			ImageIcon TempIcon = new ImageIcon(file.getPath());
			if (TempIcon == null)
			{
				Thumbnail.setIcon(null);
				return;
			}
			
			int ThumbnailSize = 64;
			int Width = TempIcon.getIconWidth();
			int Height = TempIcon.getIconHeight();
			if (Width > ThumbnailSize || Height > ThumbnailSize)
			{
				Image TempImg = TempIcon.getImage();
				if (Width >= Height)
					TempIcon = new ImageIcon(TempImg.getScaledInstance(ThumbnailSize,-1,Image.SCALE_DEFAULT));
				else
					TempIcon = new ImageIcon(TempImg.getScaledInstance(-1,ThumbnailSize,Image.SCALE_DEFAULT));
			}
			
			Thumbnail.setIcon(TempIcon);
		}
	}
}

