//
//  ImageSaveFileHandler.java
//  CBSim
//
//  Created by Loren Petrich on 5/20/08.
//  Copyright 2008 __MyCompanyName__. All rights reserved.
//

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;
import javax.imageio.*;
import javax.imageio.stream.*;
import javax.imageio.plugins.jpeg.*;
import javax.swing.*;
import javax.swing.filechooser.*;

// Handles writing of image files with a file chooser
public class ImageSaveFileHandler
{
	private JFileChooser Chooser;

	// Output image file formats;
	private JRadioButton ImgFmt_PNG, ImgFmt_JPEG;

	// JPEG Quality factor and display;
	// that factor ranges from 0 to 100,
	// and is initially set to JPEG's default
	private int JPEG_Q = 75;
	private JLabel JPEG_Q_Display;
	
	ImageSaveFileHandler()
	{
		ImageFileChooserFilter IFCF = new ImageFileChooserFilter();
	
		Chooser = new JFileChooser();
		Chooser.addChoosableFileFilter(IFCF);
		Chooser.setDialogTitle("Save Image File");
		
		JPanel ImgFmtPanel = new JPanel();
		ImgFmtPanel.setLayout(new BoxLayout(ImgFmtPanel, BoxLayout.Y_AXIS));
		int BSize = 8;
		ImgFmtPanel.setBorder(BorderFactory.createEmptyBorder(BSize,BSize,BSize,BSize));
		ButtonGroup Grp = new ButtonGroup();
		
		ImgFmtPanel.add(new JLabel("Format:"));
		
		ImgFmt_PNG = new JRadioButton("PNG",false);
		ImgFmtPanel.add(ImgFmt_PNG);
		Grp.add(ImgFmt_PNG);
		
		ImgFmt_JPEG = new JRadioButton("JPEG",true);
		ImgFmtPanel.add(ImgFmt_JPEG);
		Grp.add(ImgFmt_JPEG);
		
		JButton JPEG_Q_Button = new JButton("Quality");
		JPEG_Q_Button.setToolTipText("JPEG quality / file-size factor");
		JPEG_Q_Button.addActionListener(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					Update_JPEG_Quality();
				}
			}
		);
		ImgFmtPanel.add(JPEG_Q_Button);
		
		JPEG_Q_Display = new JLabel("");
		Update_JPEG_Quality_Display();
		ImgFmtPanel.add(JPEG_Q_Display);
		
		Chooser.setAccessory(ImgFmtPanel);
	}

	public File GetFile()
	{
		int ReturnValue = Chooser.showSaveDialog(null);
		return (ReturnValue == JFileChooser.APPROVE_OPTION) ?
			Chooser.getSelectedFile() :
				null;
	}
	
	public void SaveImage(BufferedImage Img) throws IOException
	{
		File file = GetFile();
		if (file == null) return;
		
		if (ImgFmt_PNG.isSelected())
		{
			ImageIO.write(Img,"png",file);
		}
		else if (ImgFmt_JPEG.isSelected())
		{ 
			Iterator<ImageWriter> IWIter = ImageIO.getImageWritersByFormatName("jpg");
			ImageWriter IW = IWIter.next();
			
			JPEGImageWriteParam Param = (JPEGImageWriteParam)IW.getDefaultWriteParam();
			Param.setCompressionMode(ImageWriteParam.MODE_EXPLICIT);
			Param.setOptimizeHuffmanTables(true);
			Param.setCompressionQuality(JPEG_Q/((float)100));
			
			ImageOutputStream IOS = ImageIO.createImageOutputStream(file);
			IW.setOutput(IOS);
			IW.write(null, new IIOImage(Img, null, null), Param);
		}
	}
	
	private void Update_JPEG_Quality_Display()
	{
		JPEG_Q_Display.setText("Q = " + JPEG_Q);
	}
	
	private void Update_JPEG_Quality()
	{
		int NewQ = JPEG_Q;
		while(true)
		{
			String NewQVal = JOptionPane.showInputDialog(null,
				"What JPEG Quality Value? (int from 0 to 100)", String.valueOf(NewQ));
			if (NewQVal == null) return;
			
			try
			{
				NewQ = Integer.parseInt(NewQVal);
				if (NewQ >= 0 && NewQ <= 100) break;
			}
			catch (Exception ex)
			{
				// Don't report it -- return with the old value
			}
		}
		JPEG_Q = NewQ;
		Update_JPEG_Quality_Display();
	}
}
