//
//  TimerDialog.java
//  CBSim
//
//  Created by Loren Petrich on 5/30/08.
//  Copyright 2008 __MyCompanyName__. All rights reserved.
//

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

// Includes a wait-timer object that start the wait when the dialog is hidden with "OK"

class TimerDialog
{
	// Includes parameters:
	//   WaitTime - in seconds
	WaitTimer WtTmr = new WaitTimer();
	
	// In seconds
	int MaxWaitTime = 10;
	
	// Persistent
	private JDialog Dlg;
	
	// Used internally
	private JTextField WaitText;
	private JCheckBox UseSounds;
	private JCheckBox UseCtdwnSounds;
	
	private boolean IsOK = false;
	
	private int WaitTime = 0;
	
	TimerDialog()
	{
		Dlg = new JDialog((JFrame)null, "Screenshot Wait Time", true);
		Dlg.setResizable(false);
		Container CPn = Dlg.getContentPane();
		JPanel Ctnts = new JPanel();
		CPn.add(Ctnts);
		
		Ctnts.setLayout(new BoxLayout(Ctnts, BoxLayout.Y_AXIS));
		int BSize = 8;
		Ctnts.setBorder(BorderFactory.createEmptyBorder(BSize,BSize,BSize,BSize));
		
		JPanel WaitPane = new JPanel();
		WaitPane.setLayout(new BoxLayout(WaitPane, BoxLayout.Y_AXIS));
		WaitPane.setBorder(BorderFactory.createTitledBorder("Wait Time"));
		WaitPane.add(new JLabel("How many seconds before taking the screenshot?"));
		
		WaitText = new JTextField(12);
		WaitText.setMaximumSize(WaitText.getPreferredSize());
		WaitPane.add(WaitText);
		
		Ctnts.add(WaitPane);
		
		JPanel SoundsPane = new JPanel();
		SoundsPane.setLayout(new BoxLayout(SoundsPane, BoxLayout.Y_AXIS));
		SoundsPane.setBorder(BorderFactory.createTitledBorder("Sounds"));
		
		UseSounds = new JCheckBox("Use them: System Beeps");
		SoundsPane.add(UseSounds);
		
		UseCtdwnSounds = new JCheckBox("During Countdown");
		SoundsPane.add(UseCtdwnSounds);
		
		Ctnts.add(SoundsPane);
		
		JPanel ButtonsPane = new JPanel();
		ButtonsPane.setLayout(new BoxLayout(ButtonsPane, BoxLayout.X_AXIS));
	
		ButtonsPane.add(Box.createHorizontalGlue());
		
		JButton CancelButton = new JButton("Cancel");
		CancelButton.addActionListener(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					Dlg.setVisible(false);
					IsOK = false;
				}
			}
		);
		ButtonsPane.add(CancelButton);
		
		JButton OK_Button = new JButton("OK");
		OK_Button.addActionListener(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					if (!GetValues()) return;
					
					WtTmr.WaitTime = WaitTime;
					WtTmr.UseSounds = UseSounds.isSelected();
					WtTmr.UseCtdwnSounds = UseCtdwnSounds.isSelected();
					Dlg.setVisible(false);
					IsOK = true;
				}
			}
		);
		ButtonsPane.add(OK_Button);
		
		Ctnts.add(ButtonsPane);
		
		JRootPane RtPn = Dlg.getRootPane();
		RtPn.setDefaultButton(OK_Button);
		KeyStroke EscapeKeystroke = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0, false);
		RtPn.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(EscapeKeystroke, "ESCAPE");
		RtPn.getActionMap().put("ESCAPE",
			new AbstractAction()
			{
				public void actionPerformed(ActionEvent e)
				{
					Dlg.setVisible(false);
				}
			}
		);
		
		Dlg.pack();
	}
	
	// Returns true for OK, false for cancel
	boolean show()
	{
		IsOK = false;
		WaitTime = WtTmr.WaitTime;
		UseSounds.setSelected(WtTmr.UseSounds);
		UseCtdwnSounds.setSelected(WtTmr.UseCtdwnSounds);
		
		WaitText.setText(String.valueOf(WaitTime));
		
		Dlg.setVisible(true);
		
		if (IsOK) WtTmr.Wait();
				
		return IsOK;
	}
	
	private boolean GetValues()
	{
		String WtTxtVal = WaitText.getText();
		try
		{
			WaitTime = Integer.parseInt(WtTxtVal);
		}
		catch (Exception ex)
		{
			ShowError(String.format("Not an integer: %s",WtTxtVal));
			return false;
		}
		
		if (WaitTime < 0)
		{
			ShowError(String.format("Can't do time travel; should be nonnegative: %d",WaitTime));
			WaitTime = 0;
			WaitText.setText(String.valueOf(WaitTime));
			return false;
		}
		
		if (WaitTime > MaxWaitTime)
		{
			String Msg = String.format("Wait time %d > %d seconds; continue anyway?",
				WaitTime, MaxWaitTime);
			int rc = JOptionPane.showConfirmDialog(null, Msg, "CBSim Message",
				JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE); 
			if (rc != JOptionPane.YES_OPTION) return false;
		}
		
		return true;
	}

	static void ShowError(String ErrStr)
	{
		JOptionPane.showMessageDialog(null, ErrStr, "CBSim Message", JOptionPane.ERROR_MESSAGE);
	}
}
